use core::mem;

use alloc::{string::String, sync::Arc};
use blinkcast::alloc::channel;
use microlang::Context;
use spin::Mutex;

use crate::{
	dev::console::out,
	hard::keyboard::{keyboard, Code, Key},
	print, println,
};

pub fn main() {
	let (tx, mut rx) = channel(1024);

	let buff = Arc::new(Mutex::new(String::new()));
	keyboard().on_code(move |c| {
		let mut buff = buff.lock();
		match c {
			Code::Down(Key::Char(c)) => {
				print!("{c}");
				if c == '\n' {
					let mut new = String::new();
					mem::swap(&mut new, &mut buff);
					tx.send(new);
				} else {
					buff.push(c);
				}
			}
			Code::Down(Key::BackSpace) => {
				out().correct();
				buff.pop();
			}
			_ => (),
		}
	});

	let mut context = Context::empty();

	print!("> ");
	loop {
		if let Some(line) = rx.recv() {
			match context.eval(line) {
				Ok((_, value)) => println!("{}", value.serialize()),
				Err(err) => println!("Error: {err:?}"),
			}
			print!("> ");
		}
	}
}
